import numpy as np
import matplotlib
import math
import matplotlib.pyplot as plt
from matplotlib import ticker, cm
from matplotlib.gridspec import GridSpec
from matplotlib import rc
from mpl_toolkits.axes_grid1.inset_locator import inset_axes

# ==========================================================================================

def ReadData():

	global Nx,Ny
	global X,Y,DSD,AVG,COUNT

	bff1 = np.loadtxt('T05RH50/X.dat')
	bff3 = np.loadtxt('T05RH90/X.dat')
	bff5 = np.loadtxt('T20RH50/X.dat')
	bff7 = np.loadtxt('T20RH90/X.dat')

	bff2 = np.loadtxt('T05RH50/Y.dat')
	bff4 = np.loadtxt('T05RH90/Y.dat')
	bff6 = np.loadtxt('T20RH50/Y.dat')
	bff8 = np.loadtxt('T20RH90/Y.dat')

	Nx   = np.int(bff1[0])
	Ny   = np.int(bff2[0])

	X      = np.zeros((4,Nx)   ,dtype=np.float64)
	Y      = np.zeros((4,Ny)   ,dtype=np.float64)
	DSD    = np.zeros((4,Ny,Nx),dtype=np.float64)

	X[0,:] = np.float64(bff1[1:len(bff1)])*5E-4#Transformation from the non-dimensional time to dimensional one [s]
	X[1,:] = np.float64(bff3[1:len(bff3)])*5E-4
	X[2,:] = np.float64(bff5[1:len(bff5)])*5E-4
	X[3,:] = np.float64(bff7[1:len(bff7)])*5E-4

	Y[0,:] = np.float64(bff2[1:len(bff2)])
	Y[1,:] = np.float64(bff4[1:len(bff4)])
	Y[2,:] = np.float64(bff6[1:len(bff6)])
	Y[3,:] = np.float64(bff8[1:len(bff8)])

	# -----------

	bff1 = np.loadtxt('T05RH50/DSD.dat')
	bff2 = np.loadtxt('T05RH90/DSD.dat')
	bff3 = np.loadtxt('T20RH50/DSD.dat')
	bff4 = np.loadtxt('T20RH90/DSD.dat')

	APPO1 = bff1[:,:]
	APPO2 = bff2[:,:]
	APPO3 = bff3[:,:]
	APPO4 = bff4[:,:]

	APPO1 = APPO1.transpose()
	APPO2 = APPO2.transpose()
	APPO3 = APPO3.transpose()
	APPO4 = APPO4.transpose()

	DSD[0,:,:] = APPO1
	DSD[1,:,:] = APPO2
	DSD[2,:,:] = APPO3
	DSD[3,:,:] = APPO4

	print('Reading Finish ......')

	return

# =============================================================================================

def PlotData():

	FS = 20
	rc('text',usetex=True)
	rc('font',size=FS)

	x_min = 0
	x_max = 2.9
	dx    = 0.5

	y_min = 1.4
	y_max = 5.0E2

	t     = np.linspace(x_min,x_max,1000)

	# ----------- Well's theory

	rho_d   = 998
	mu_g    = 1.9931E-5
	Sh      = 2
	Sc      = 0.6627
	Y_vs    = 0.035963#H2O vapor mass fraction on droplets surface (Td=35)

	Yvs_1_r = 0.004236#H2O vapor mass fraction on droplets surface (Td=1)
	Yvs_2_r = 0.005313#H2O vapor mass fraction on droplets surface (Td=4.5)
	Yvs_3_r = 0.00974#H2O vapor mass fraction on droplets surface (Td=13.5)
	Yvs_4_r = 0.0137478#H2O vapor mass fraction on droplets surface (Td=19)

	Yv_1    = 0.00282575320#H2O vapor mass fraction (T05RH50)
	Yv_2    = 0.00509298918#H2O vapor mass fraction (T05RH90)
	Yv_3    = 0.00742096562#H2O vapor mass fraction (T20RH50)
	Yv_4    = 0.01340358512#H2O vapor mass fraction (T20RH90)

	k_evp_1   = Sh*mu_g*math.log((1-Yv_1)/(1-Y_vs))/(Sc*rho_d)
	k_evp_2   = Sh*mu_g*math.log((1-Yv_2)/(1-Y_vs))/(Sc*rho_d)
	k_evp_3   = Sh*mu_g*math.log((1-Yv_3)/(1-Y_vs))/(Sc*rho_d)
	k_evp_4   = Sh*mu_g*math.log((1-Yv_4)/(1-Y_vs))/(Sc*rho_d)

	k_evp_5_r = Sh*mu_g*math.log((1-Yv_1)/(1-Yvs_1_r))/(Sc*rho_d)
	k_evp_6_r = Sh*mu_g*math.log((1-Yv_2)/(1-Yvs_2_r))/(Sc*rho_d)
	k_evp_7_r = Sh*mu_g*math.log((1-Yv_3)/(1-Yvs_3_r))/(Sc*rho_d)
	k_evp_8_r = Sh*mu_g*math.log((1-Yv_4)/(1-Yvs_4_r))/(Sc*rho_d)

	D_evp_1   = 2*1E6*np.sqrt(4.225E-13+t*k_evp_1)#equation: r^2_{d,0}=(6.5E-7)^2 + t * k_evp
	D_evp_2   = 2*1E6*np.sqrt(4.225E-13+t*k_evp_2)
	D_evp_3   = 2*1E6*np.sqrt(4.225E-13+t*k_evp_3)
	D_evp_4   = 2*1E6*np.sqrt(4.225E-13+t*k_evp_4)

	D_evp_5_r = 2*1E6*np.sqrt(4.225E-13+t*k_evp_5_r)#equation: r^2_{d,0}=(6.5E-7)^2 + t * k_evp
	D_evp_6_r = 2*1E6*np.sqrt(4.225E-13+t*k_evp_6_r)
	D_evp_7_r = 2*1E6*np.sqrt(4.225E-13+t*k_evp_7_r)
	D_evp_8_r = 2*1E6*np.sqrt(4.225E-13+t*k_evp_8_r)

	# ----------- End Well's theory & Start layout

	gs  = GridSpec(2,4,width_ratios=[0.47,0.47,0.01,0.05],height_ratios=[0.5,0.5])

	fig = plt.figure(figsize=(16,9))

	ax1 = fig.add_subplot(gs[0,0])
	ax2 = fig.add_subplot(gs[0,1])
	ax3 = fig.add_subplot(gs[1,0])
	ax4 = fig.add_subplot(gs[1,1])
	cax = fig.add_subplot(gs[:,2])#colorbar axes

	fig.subplots_adjust(left=0.08,bottom=0.1,right=0.985,top=0.985,wspace=0.3,hspace=0.25)

	ax1.set_xlim(x_min,x_max)
	ax2.set_xlim(x_min,x_max)
	ax3.set_xlim(x_min,x_max)
	ax4.set_xlim(x_min,x_max)

	ax1.set_ylim(y_min,y_max)
	ax2.set_ylim(y_min,y_max)
	ax3.set_ylim(y_min,y_max)
	ax4.set_ylim(y_min,y_max)

	xarr = np.arange(x_min,x_max,dx)
	ax1.set_xticks(xarr)
	ax2.set_xticks(xarr)
	ax3.set_xticks(xarr)
	ax4.set_xticks(xarr)
	ax1.set_xticklabels([r"${:5.1f}$".format(x) for x in xarr])
	ax2.set_xticklabels([r"${:5.1f}$".format(x) for x in xarr])
	ax3.set_xticklabels([r"${:5.1f}$".format(x) for x in xarr])
	ax4.set_xticklabels([r"${:5.1f}$".format(x) for x in xarr])

	yarr = np.arange(y_min,y_max)
	ax1.set_yticks(yarr)
	ax2.set_yticks(yarr)
	ax3.set_yticks(yarr)
	ax4.set_yticks(yarr)
	ax1.set_yticklabels([r"${:5.1f}$".format(y) for y in yarr])
	ax2.set_yticklabels([r"${:5.1f}$".format(y) for y in yarr])
	ax3.set_yticklabels([r"${:5.1f}$".format(y) for y in yarr])
	ax4.set_yticklabels([r"${:5.1f}$".format(y) for y in yarr])

	ax1.set_yscale('log')
	ax2.set_yscale('log')
	ax3.set_yscale('log')
	ax4.set_yscale('log')

	ax1.tick_params(axis='x',pad=8)
	ax2.tick_params(axis='x',pad=8)
	ax3.tick_params(axis='x',pad=8)
	ax4.tick_params(axis='x',pad=8)

	ax1.tick_params(axis='y',pad=8)
	ax2.tick_params(axis='y',pad=8)
	ax3.tick_params(axis='y',pad=8)
	ax4.tick_params(axis='y',pad=8)

	ax1.set_xlabel(r"$t_e\ (s)$")
	ax2.set_xlabel(r"$t_e\ (s)$")
	ax3.set_xlabel(r"$t_e\ (s)$")
	ax4.set_xlabel(r"$t_e\ (s)$")

	ax1.set_ylabel(r"$D_{d,0}\ ( \mu m )$")
	ax2.set_ylabel(r"$D_{d,0}\ ( \mu m )$")
	ax3.set_ylabel(r"$D_{d,0}\ ( \mu m )$")
	ax4.set_ylabel(r"$D_{d,0}\ ( \mu m )$")

	ax1.text(1.55,300,r"$T=5^\circ C RH=50\%$")
	ax2.text(1.55,300,r"$T=5^\circ C RH=90\%$")
	ax3.text(1.55,300,r"$T=20^\circ C RH=50\%$")
	ax4.text(1.55,300,r"$T=20^\circ C RH=90\%$")

	ax1.grid(linestyle='--',alpha=0.6)
	ax2.grid(linestyle='--',alpha=0.6)
	ax3.grid(linestyle='--',alpha=0.6)
	ax4.grid(linestyle='--',alpha=0.6)

	# ----------- End layout & Start plot

	AVG    = np.zeros((4,Ny)   ,dtype=np.float64)
	COUNT  = np.zeros((4,Ny)   ,dtype=np.float64)

	CN    = 9

	for j in range(Nx):

		for i in range(Ny):

			for k in range(4):

				if(DSD[k,i,j]>1E-10):

					AVG[k,i]   = AVG[k,i] + DSD[k,i,j] * X[k,j]

					COUNT[k,i] = COUNT[k,i] + DSD[k,i,j]

	for i in range(Ny):

		for k in range(4):

			if(COUNT[k,i]>0.0):

				AVG[k,i] = AVG[k,i] / COUNT[k,i]

	#print(DSD[1,40,:])

	for j in range(Nx):

		for i in range(Ny):

			for k in range(4):

				if(DSD[k,i,j]>0.01):

					DSD[k,i,j]=0.01

				if(DSD[k,i,j]<1E-10):

					DSD[k,i,j]=1E-10

	V_min = 1.0E-6
	V_max = 1.0E-2

	CL_min = np.log10(V_min)
	CL_max = np.log10(V_max)

	CL = 10**(np.linspace(CL_min,CL_max,CN))

	CS = ax1.contourf(X[0,:],Y[0,:],DSD[0,:,:],CL,norm=matplotlib.colors.LogNorm(),cmap=plt.cm.cividis)
	CS = ax2.contourf(X[1,:],Y[1,:],DSD[1,:,:],CL,norm=matplotlib.colors.LogNorm(),cmap=plt.cm.cividis)
	CS = ax3.contourf(X[2,:],Y[2,:],DSD[2,:,:],CL,norm=matplotlib.colors.LogNorm(),cmap=plt.cm.cividis)
	CS = ax4.contourf(X[3,:],Y[3,:],DSD[3,:,:],CL,norm=matplotlib.colors.LogNorm(),cmap=plt.cm.cividis)

	cbar = fig.colorbar(CS,ticks=CL,cax=cax,format=r'$%.1e$')
	cbar.set_label(r'$JPDF(t_e,D_{d,0})$',labelpad=-125)#,x=0.8)

	ax1.plot(AVG[0,:],Y[0,:],'o',color='black',markevery=0.04,markersize=10,fillstyle='none')
	ax2.plot(AVG[1,:],Y[1,:],'o',color='black',markevery=10,markersize=10,fillstyle='none')
	ax3.plot(AVG[2,:],Y[2,:],'o',color='black',markevery=0.04,markersize=10,fillstyle='none')
	ax4.plot(AVG[3,:],Y[3,:],'o',color='black',markevery=0.04,markersize=10,fillstyle='none')

	ax1.plot(t,D_evp_1,'-',color='red',markevery=100,markersize=10,fillstyle='none')
	ax2.plot(t,D_evp_2,'-',color='red',markevery=100,markersize=10,fillstyle='none')
	ax3.plot(t,D_evp_3,'-',color='red',markevery=100,markersize=10,fillstyle='none')
	ax4.plot(t,D_evp_4,'-',color='red',markevery=100,markersize=10,fillstyle='none')

	ax1.plot(t,D_evp_5_r,'--',color='blue',markevery=0.04,markersize=6,fillstyle='none')
	ax2.plot(t,D_evp_6_r,'--',color='blue',markevery=0.04,markersize=6,fillstyle='none')
	ax3.plot(t,D_evp_7_r,'--',color='blue',markevery=0.04,markersize=6,fillstyle='none')
	ax4.plot(t,D_evp_8_r,'--',color='blue',markevery=0.04,markersize=6,fillstyle='none')

	ax1.legend([r"$\langle t_e \rangle(D_{d,0})$",r"Wells theory~($T_d \equiv T_{jet}$)",r"Revised $D^2$-law"],loc='upper left',fontsize=15,ncol=1)
	ax2.legend([r"$\langle t_e \rangle(D_{d,0})$",r"Wells theory~($T_d \equiv T_{jet}$)",r"Revised $D^2$-law"],loc='upper left',fontsize=15,ncol=1)
	ax3.legend([r"$\langle t_e \rangle(D_{d,0})$",r"Wells theory~($T_d \equiv T_{jet}$)",r"Revised $D^2$-law"],loc='upper left',fontsize=15,ncol=1)
	ax4.legend([r"$\langle t_e \rangle(D_{d,0})$",r"Wells theory~($T_d \equiv T_{jet}$)",r"Revised $D^2$-law"],loc='upper left',fontsize=15,ncol=1)

	plt.savefig('figure2.png',dpi=300)

	plt.show()

	return

# =============================================================================================

ReadData()
PlotData()
