import numpy as np
import math
import matplotlib.pyplot as plt
from matplotlib import rc

# ==========================================================================================================

def SetParams():

	global Td_org,p_amb,T_amb
	global RH_min,RH_max,RH_int
	global Nu,Sh
	global ps_ref,Ts_ref
	global M_a,M_v
	global rho_l,DH_vap
	global Gr,R_uni,T0K
	global R_a,R_v
	global TK_amb,TKd_org
	global mu_a,kappa_a,cp_a
	global mu_v,kappa_v,cp_v
	global Phi_va,Phi_av
	global D_amb
	global Height
	global U_spk,U_snz
	global U_cgh,Tm_inj,R_in

	fid = open('./fig5.dat')#papameters

	lin = fid.readlines()

	c = 0

	for l in lin:

		c = c+1

		if c == 3 : Td_org = np.float64(l.split("\t")[0])
		if c == 4 : p_amb  = np.float64(l.split("\t")[0])
		if c == 5 : T_amb  = np.float64(l.split("\t")[0])
		if c == 6 : RH_min = np.float64(l.split("\t")[0])
		if c == 7 : RH_max = np.float64(l.split("\t")[0])
		if c == 8 : RH_int = np.float64(l.split("\t")[0])
		if c == 9 : Nu     = np.float64(l.split("\t")[0])
		if c == 10: Sh     = np.float64(l.split("\t")[0])

		if c == 13: ps_ref = np.float64(l.split("\t")[0])
		if c == 14: Ts_ref = np.float64(l.split("\t")[0])

		if c == 17: M_a    = np.float64(l.split("\t")[0])
		if c == 20: M_v    = np.float64(l.split("\t")[0])

		if c == 23: rho_l  = np.float64(l.split("\t")[0])
		if c == 24: DH_vap = np.float64(l.split("\t")[0])

		if c == 27: Gr     = np.float64(l.split("\t")[0])
		if c == 28: R_uni  = np.float64(l.split("\t")[0])
		if c == 29: T0K    = np.float64(l.split("\t")[0])
		if c == 30: Height = np.float64(l.split("\t")[0])

		if c == 33: U_spk  = np.float64(l.split("\t")[0])
		if c == 34: U_cgh  = np.float64(l.split("\t")[0])
		if c == 35: U_snz  = np.float64(l.split("\t")[0])
		if c == 36: U_snz  = np.float64(l.split("\t")[0])
		if c == 37: Tm_inj = np.float64(l.split("\t")[0])
		if c == 38: R_in   = np.float64(l.split("\t")[0])

	R_a = R_uni/M_a
	R_v = R_uni/M_v

	RH_min  = RH_min/100.0
	RH_max  = RH_max/100.0
	RH_int  = RH_int/100.0

	TK_amb  = T_amb+T0K
	TKd_org = Td_org+T0K

	mu_a    = (-9.8601E-1    + 9.080125E-2   *TK_amb - 1.17635575E-4 *TK_amb**2 + 1.2349703E-7  *TK_amb**3 - 5.7971299E-11*TK_amb**4) * 1.0E-6
	kappa_a = ( -2.276501E-3 + 1.2598485E-4  *TK_amb - 1.4815235E-7  *TK_amb**2 + 1.73550646E-10*TK_amb**3 - 1.066657E-13 *TK_amb**4  + 2.47663035E-17 *TK_amb**5)
	cp_a    = ( 1.03409      - 2.84887E-4    *TK_amb + 7.816818E-7   *TK_amb**2 - 4.970786E-10  *TK_amb**3 + 1.077024E-13 *TK_amb**4) * 1.0E3

	D_amb   = -2.775E-6      + 4.479E-8      *TK_amb + 1.656E-10     *TK_amb**2

	mu_v    = (8.058131868E1 + 4.000549451E-1*T_amb) * 1.0E-6
	kappa_v = (1.761758242E1 + 5.558941059E-2*T_amb  + 1.663336663E-4*T_amb**2) * 1.0E-3
	cp_v    = (1.86910989    - 2.578421578E-4*T_amb  + 1.941058941E-5*T_amb**2) * 1.0E3

	Phi_av  = (1. + M_a / M_v)**(-0.5) * (1. + (mu_a / mu_v)**0.5* (M_v / M_a)**0.25 )**2. * math.sqrt(2.) / 4.
	Phi_va  = (1. + M_v / M_a)**(-0.5) * (1. + (mu_v / mu_a)**0.5* (M_a / M_v)**0.25 )**2. * math.sqrt(2.) / 4.

	return

# ==========================================================================================================

def SatVapPress(TK):

	p_vs = ps_ref*np.exp(DH_vap/R_v*(1.0/Ts_ref-1.0/TK))

	return p_vs

# ==========================================================================================================

def VapFrac(RH,p_vs,p):

	global M_a,M_v

	X_v = RH*p_vs/p

	Y_v = X_v/(X_v+(1.0-X_v)*M_a/M_v)

	return X_v,Y_v

# ==========================================================================================================

def f(TK,cp_amb,Pr,Sc):

	global p_amb
	global Nu
	global Sh
	global DH_v
	global TK_amb

	p_vs = SatVapPress(TK)

	[X_vs,Y_vs] = VapFrac(1.0,p_vs,p_amb)

	Hm   = np.log((1.0-Y_v)/(1.0-Y_vs))

	F    = Nu/Pr*cp_amb*(TK_amb-TK)-DH_vap*Sh/Sc*Hm

	return F

# ==========================================================================================================

def DropRegTemp(cp_amb,Pr,Sc):

	global TK_amb

	A = TK_amb*0.7
	B = TK_amb*1.3

	imax = 100
	toll = 1.0e-10

	i = 0
	r = 1000

	while ( (i < imax) and (abs(r) > toll)):

		i = i+1

		C = (A+B)/2.0

		r = f(C,cp_amb,Pr,Sc)

		dum_C = f(C,cp_amb,Pr,Sc)
		dum_A = f(A,cp_amb,Pr,Sc)

		if (np.sign(dum_C) == np.sign(dum_A)):

			A = C

		else:

			B = C

	T_d = C

	return T_d

# ==========================================================================================================
#
#  MAIN

SetParams()

RH = np.arange(RH_min,RH_max+1E-3,RH_int)
RH = np.delete(RH,8)
RH = np.delete(RH,7)
RH = np.delete(RH,6)
RH = np.delete(RH,4)
RH = np.delete(RH,3)
RH = np.delete(RH,2)
RH = np.delete(RH,0)

zf    	= np.linspace(0,10,1000)
d    	= np.logspace(-1,5.5,endpoint=False,num=1000,base=np.e)
Tm	= np.zeros((len(RH),1000),dtype=np.float64)
Z_spk	= np.zeros((len(RH),1000),dtype=np.float64)
Z_cgh	= np.zeros((len(RH),1000),dtype=np.float64)
Z_snz	= np.zeros((len(RH),1000),dtype=np.float64)
Rk_spk	= np.zeros((len(RH),1000),dtype=np.float64)
Rk_cgh	= np.zeros((len(RH),1000),dtype=np.float64)
Rk_snz  = np.zeros((len(RH),1000),dtype=np.float64)

Vol_d   = np.pi*np.power(d,3)/6.0
DSD     = np.exp(-(np.log(d)-np.log(12))**2/(2*0.49))/(0.7*d*np.sqrt(2*np.pi))
DSD_BIN = (np.exp(0.0065)-np.exp(-0.0065))*d*0.5

Vol_tot = 0
for i in range(len(d)):
	Vol_tot = Vol_tot + Vol_d[i]*DSD[i]
	#print(d[i],np.log(d[i]/12)/0.7,Vol_d[i])
#print(np.sum(DSD*DSD_BIN),Vol_tot/np.sum(DSD))

j  = 0

for RH_amb in RH:

	# ----------------------------------------------
	# computing moisted gas properties

	p_vs = SatVapPress(TK_amb)

	[X_v,Y_v] = VapFrac(RH_amb,p_vs,p_amb)

	rho_amb   = p_amb/((Y_v/M_v+(1.0-Y_v)/M_a)*R_uni*TK_amb)

	mu_amb    = ((1. - X_v) * mu_a)    / (1. - X_v + X_v * Phi_av) + (X_v * mu_v)    / (X_v + (1. - X_v) * Phi_va)
	kappa_amb = ((1. - X_v) * kappa_a) / (1. - X_v + X_v * Phi_av) + (X_v * kappa_v) / (X_v + (1. - X_v) * Phi_va)

	M_amb     = M_a * (1. - X_v ) + M_v * X_v

	cp_amb    = cp_a * (1. - X_v) * M_a / M_amb + cp_v * X_v * M_v /M_amb

	Pr	  = mu_amb * cp_amb / kappa_amb
	Sc	  = mu_amb / (D_amb * rho_amb)

	tau_d	  = rho_l*np.power(d*1E-6,2)/mu_amb/18
	Tm_trs    = 6.0*tau_d#Transition time for drop to reach asymptotic temperature

	#print(Y_v,rho_amb,mu_amb,M_amb,cp_amb,Pr,Sc,RH_amb)

	# ----------------------------------------------
	# computing droplet properties

	TKd_crr = DropRegTemp(cp_amb,Pr,Sc)

	print([TKd_org,TK_amb,TKd_crr])

	# ----------------------------------------------
	# computing droplet constants

	p_vs = SatVapPress(TKd_crr)

	[X_vs,Y_vs] = VapFrac(1.0,p_vs,p_amb)

	Hm   = np.log((1.0-Y_v)/(1.0-Y_vs))

	check = TKd_crr+Pr/Sc*Sh/Nu*DH_vap/cp_amb*Hm-TK_amb
	print(abs(check)<1.0e-10)

	Kr_crr = mu_amb/rho_l*Sh/Sc*Hm

	p_vs = SatVapPress(TKd_org)

	[X_vs,Y_vs] = VapFrac(1.0,p_vs,p_amb)

	Hm   = np.log((1.0-Y_v)/(1.0-Y_vs))

	Kr_org   = mu_amb/rho_l*Sh/Sc*Hm

	Kr_set   = 9.0*Height*mu_amb/2.0/(rho_l-rho_amb)/Gr

	#print(Kr_crr,Kr_set)

	# ----------------------------------------------
	# computing evaporation-falling curves

	T_evp = (np.power(d*0.5*1E-6,2)-0.0-Tm_trs*(Kr_org-Kr_crr))/Kr_crr


	rd_set	     = 0.0#rd^3_set is one order smaller than rd^3_0; So we directly set rd_set as 0
	rd_star	     = np.power(0.5*(np.power(d*0.5*1E-6,3)+np.power(rd_set,3)),1.0/3.0)
	Re_star      = 4*(rho_l-rho_amb)*rho_amb*Gr*np.power(rd_star,3)/np.power(mu_amb,2)/9
	Kr_set_r     = (1+0.15*np.power(Re_star,0.687))*9.0*Height*mu_amb/2.0/(rho_l-rho_amb)/Gr - (Kr_org-Kr_crr)*np.power(Tm_trs,2)*0.5
	T_set       = ((np.power(d*0.5*1E-6,2)-(Kr_org-Kr_crr)*Tm_trs)-np.sqrt(np.power(np.power(d*0.5*1E-6,2)-(Kr_org-Kr_crr)*Tm_trs,2)-2*Kr_crr*Kr_set_r))/(Kr_crr)

	for i in range(len(d)):

		if(math.isnan(T_set[i])or T_set[i]>T_evp[i]):

			Tm[j,i] = T_evp[i]
		else:

			Tm[j,i] = T_set[i]

	# ----------------------------------------------
	# computing flight distances

	for i in range(len(d)):

		if(Tm[j,i]<Tm_inj):

			Z_spk[j,i]	= 2.0*np.sqrt(6.0*U_spk*R_in*Tm[j,i])
			Z_cgh[j,i]	= 2.0*np.sqrt(6.0*U_cgh*R_in*Tm[j,i])
			Z_snz[j,i]	= 2.0*np.sqrt(6.0*U_snz*R_in*Tm[j,i])

		else:

			Z_spk[j,i]	= 2.0*np.sqrt(6.0*U_spk*R_in)*Tm_inj**0.25*np.power(Tm[j,i],0.25)
			Z_cgh[j,i]	= 2.0*np.sqrt(6.0*U_cgh*R_in)*Tm_inj**0.25*np.power(Tm[j,i],0.25)
			Z_snz[j,i]	= 2.0*np.sqrt(6.0*U_snz*R_in)*Tm_inj**0.25*np.power(Tm[j,i],0.25)

	j = j + 1

# ----------------------------------------------
# computing the relative risk at different distances

for k in range(len(zf)):

	for i in range(len(d)):

		for j in range(len(RH)):

			if Z_spk[j,i]   > zf[k]:

				Rk_spk[j,k]   = Rk_spk[j,k] + Vol_d[i]*DSD[i]/Vol_tot

			if Z_cgh[j,i]   > zf[k]:

				Rk_cgh[j,k]   = Rk_cgh[j,k] + Vol_d[i]*DSD[i]/Vol_tot

			if Z_snz[j,i]   > zf[k]:

				Rk_snz[j,k]   = Rk_snz[j,k] + Vol_d[i]*DSD[i]/Vol_tot

# ----------------------------------------------
# plot
#
FS = 30
rc('text',usetex=True)
rc('font',size=FS)

y_min = 1.0E-6
y_max = 2.0

x_min = 0.0
x_max = 8.0
dx    = 1.0

fig,ax = plt.subplots(figsize=(12,9))

fig.subplots_adjust(left=0.11,bottom=0.12,right=0.97,top=0.985,wspace=0.0,hspace=0.0)

ax.set_xlim(x_min,x_max)
ax.set_ylim(y_min,y_max)

xarr = np.arange(x_min,x_max+1E-10,dx)
ax.set_xticks(xarr)
ax.set_xticklabels([r"${:5.0f}$".format(x) for x in xarr])

yarr = np.arange(y_min,y_max)
ax.set_yticks(yarr)
ax.set_yticklabels([r"${:5.0f}$".format(y) for y in yarr])

ax.set_yscale('log')

ax.tick_params(axis='x',pad=8)
ax.tick_params(axis='y',pad=8)

#plt.axhline(y=0.0176, color='r', linestyle='-')
#plt.axhline(y=0.00176, color='r', linestyle='-')
#plt.axhline(y=6.e-5, color='r', linestyle='-')
#plt.axhline(y=6.e-6, color='r', linestyle='-')
from matplotlib.patches import Rectangle
ax.add_patch(Rectangle((0, 0.00176), 8, 0.0158,alpha=0.5,color='grey'))
ax.add_patch(Rectangle((0, 6.1728e-06), 8, 5.55e-5,alpha=0.5,color='grey'))


#ax.set_xlabel(r"Distance "+r"$L_d\ [ m ]$")
#ax.set_ylabel(r"Normalized infectious risk")

#ax.text(0.2,2E-1,r"$T_{\infty}=20^\circ C$")
#ax.text(0.2,6E-2,r"$T_{0}=35^\circ C$")
#ax.text(0.2,2E-2,r"$R_{0}=0.015m$")
#ax.text(1.0,2E-5 ,r"$U_{spk}=1m/s$")
#ax.text(3.0,1E-3,r"$U_{cgh}=10m/s$")
#ax.text(6.0,1E-1,r"$U_{snz}=30m/s$")

ax.set_xlabel(r"Distance "+r"$L_d\ [ m ]$")
ax.set_ylabel(r"Normalized virus exposure")


ax.grid(linestyle='--',alpha=0.6)

# ----------- End layout & Start plot

colors = ["red","blue","black","red","purple","brown","pink","gray","olive","cyan"]
ncolor = 0

RH_val = ["10\%","50\%","90\%"]

for i in range(len(RH)):
	ax.plot(zf,Rk_spk[i,:]*2.5E-4,'--', color=colors[ncolor],markevery=100,markersize=10,fillstyle='none',linewidth=2,label=r'Speaking')
	ncolor = ncolor + 1

ncolor = 0
for i in range(len(RH)):
	ax.plot(zf,Rk_cgh[i,:]*5E-3,':', color=colors[ncolor],markevery=100,markersize=10,fillstyle='none',linewidth=3,label=r'Coughing')
	ncolor = ncolor + 1

ncolor = 0
for i in range(len(RH)):
	ax.plot(zf,Rk_snz[i,:],'-',color=colors[ncolor],markevery=100,markersize=10,fillstyle='none',label=r'Sneezing'+r' @ RH='+RH_val[i]+r'')
	ncolor = ncolor + 1

ax.legend(loc='lower right',fontsize=20,ncol=3,columnspacing=0.5,handletextpad=0.1,framealpha=1.0)
plt.setp(ax.spines.values(), linewidth=2)
ax.xaxis.set_tick_params(width=2)
ax.yaxis.set_tick_params(width=2)
plt.savefig('figure5.png',dpi=300)

plt.show()
